/****************************************************************************
 *
 * Bruch1: Erste Version der Klasse Bruch
 *
 *  - Beispielprogramm zum Buch "Objektorientiertes Programmieren in C++",
 *    von Nicolai Josuttis, erschienen bei Addison-Wesley, ISBN 3-89319-637-4
 *
 *  - Abschnitt 3.2: Die erste Klasse: Bruch
 *  
 ****************************************************************************/

/****************************************************************************
 *
 * Copyright 1994 by Nicolai Josuttis and Addison-Wesley.
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software for personal
 * and educational use is hereby granted without fee, provided that the above
 * copyright notice appear in all copies and that both that copyright notice
 * and this permission notice appear in supporting documentation, and that
 * the names of Addison-Wesley or the authors are not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission.  Addison-Wesley and the authors make no
 * representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * ADDISON-WESLEY AND THE AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL ADDISON-WESLEY OR THE AUTHORS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 ****************************************************************************/

// da die meisten Compiler "bool" (noch) nicht koennen (siehe Seite 43):
typedef int bool;
#define true 1
#define false 0



/****************************************************************************
 *
 *	Bruch.h
 *
 ****************************************************************************/

#ifndef _Bruch_h
#define _Bruch_h

/* Standard-Header-Dateien einbinden
 */
#include <stdio.h>
#include <stdlib.h>
/* Klasse Bruch
 */
class Bruch {

  /* privat: kein Zugriff von aussen
   */
  private:       
    int zaehler;
    int nenner;

  /* oeffentliche Schnittstelle
   */
  public:
    // Default-Konstruktor
    Bruch ();

    // Konstruktor aus Zaehler (int)
    Bruch (int);

    // Konstruktor aus Zaehler und Nenner (zwei int's)
    Bruch (int, int);

    // Ausgabe
    void print ();
};

#endif    /* _Bruch_h */


/****************************************************************************
 *
 *	Bruch.cc / Bruch.cpp
 *
 ****************************************************************************/

// Header-Datei mit der Klassen-Deklaration einbinden
//#include "Bruch.h"

/* Default-Konstruktor
 */
Bruch::Bruch ()
{
    // Bruch mit 0 initialisieren
    zaehler = 0;
    nenner  = 1;
}
/* Konstruktor aus ganzer Zahl
 */
Bruch::Bruch (int z)
{
    // Bruch mit z 1tel initialisieren
    zaehler = z;
    nenner  = 1;
}

/* Konstruktor aus Zaehler und Nenner
 */
Bruch::Bruch (int z, int n)
{
    /* Zaehler und Nenner, wie uebergeben, initialisieren
     *  - 0 als Nenner ist allerdings nicht erlaubt
     */
    if (n == 0) {
        // Programm mit Fehlermeldung beenden
        fprintf (stderr, "Fehler: Nenner ist 0\n");
        exit (1);
    }
    zaehler = z;
    nenner  = n;
}

/* print
 */
void Bruch::print ()
{
    printf ("%d/%d\n", zaehler, nenner);
}


/****************************************************************************
 *
 *	btest1.cc / btest1.cpp
 *
 ****************************************************************************/

// Header-Dateien fuer die verwendeten Klassen einbinden
//#include "Bruch.h"

main()
{
    Bruch x;             // Initialisierung durch Default-Konstruktor
    Bruch a(7,3);        // Initialisierung durch int/int-Konstruktor

    // Bruch a ausgeben
    a.print();

    // Bruch a wird an Bruch x zugewiesen
    x = a;

    // 1000 in einen Bruch umwandeln und a zuweisen
    a = Bruch(1000);

    // x und a ausgeben
    x.print();
    a.print();
}

