/****************************************************************************
 *
 * Bruch4: Klasse Bruch mit Referenzen
 *
 *  - Beispielprogramm zum Buch "Objektorientiertes Programmieren in C++",
 *    von Nicolai Josuttis, erschienen bei Addison-Wesley, ISBN 3-89319-637-4
 *
 *  - Abschnitt 3.5: Referenzen und Konstanten
 *  
 ****************************************************************************/

/****************************************************************************
 *
 * Copyright 1994 by Nicolai Josuttis and Addison-Wesley.
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software for personal
 * and educational use is hereby granted without fee, provided that the above
 * copyright notice appear in all copies and that both that copyright notice
 * and this permission notice appear in supporting documentation, and that
 * the names of Addison-Wesley or the authors are not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission.  Addison-Wesley and the authors make no
 * representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * ADDISON-WESLEY AND THE AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL ADDISON-WESLEY OR THE AUTHORS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 ****************************************************************************/

// da die meisten Compiler "bool" (noch) nicht koennen (siehe Seite 43):
typedef int bool;
#define true 1
#define false 0



/****************************************************************************
 *
 *	Bruch.h
 *
 ****************************************************************************/

#ifndef _Bruch_h
#define _Bruch_h

// Standard-Header-Dateien einbinden
#include <stdio.h>
#include <stdlib.h>

class Bruch {

  private:
    int zaehler;
    int nenner;

  public:
    /* Default-Konstruktor, Konstruktor aus Zaehler und
     * Konstruktor aus Zaehler und Nenner
     */
    Bruch (int = 0, int = 1);

    // Ausgabe (inline definiert)
    void print () const {
        printf ("%d/%d\n", zaehler, nenner);
    }

    // Multiplikation
    Bruch operator * (const Bruch&) const;
    
    // multiplikative Zuweisung
    const Bruch& operator *= (const Bruch&);
    
    // Vergleich
    bool operator < (const Bruch&) const;
};
/* operator *
 *  - inline definiert
 */
inline Bruch Bruch::operator * (const Bruch& b) const
{
    /* Zaehler und Nenner einfach multiplizieren
     *  - kuerzen sparen wir uns
     */
    return Bruch (zaehler * b.zaehler, nenner * b.nenner);
}

#endif  /* _Bruch_h */


/****************************************************************************
 *
 *	Bruch.cc / Bruch.cpp
 *
 ****************************************************************************/

// Header-Datei der Klasse einbinden
//#include "Bruch.h"

/* Default-Konstruktor, Konstruktor aus ganzer Zahl,
 * Konstruktor aus Zaehler und Nenner
 *  - Default fuer z: 0
 *  - Default fuer n: 1
 */
Bruch::Bruch (int z, int n)
{
    /* Zaehler und Nenner, wie uebergeben, initialisieren
     *  - 0 als Nenner ist allerdings nicht erlaubt
     *  - ein negatives Vorzeichen des Nenners kommt in den Zaehler
     */
    if (n == 0) {
        fprintf (stderr, "Fehler: Nenner ist 0\n");
        exit (1);
    }
    if (n < 0) {
        zaehler = -z;
        nenner  = -n;
    }
    else {
        zaehler = z;
        nenner  = n;
    }
}

/* operator *=
 */
const Bruch& Bruch::operator *= (const Bruch& b)
{
    zaehler *= b.zaehler;
    nenner  *= b.nenner;

    // Objekt (erster Operand) wird zurueckgeliefert
    return *this;
}

/* operator <
 */
bool Bruch::operator < (const Bruch& b) const
{
    // Da die Nenner nicht negativ sein koennen reicht:
    return zaehler * b.nenner < b.zaehler * nenner;
}


/****************************************************************************
 *
 *	btest4.cc / btest4.cpp
 *
 ****************************************************************************/

// Header-Dateien fuer die verwendeten Klassen einbinden
//#include "Bruch.h"

main()
{
    // NEU: Bruch a als Konstante deklarieren
    const Bruch a(7,3);

    a.print();               // Bruch a ausgeben

    // x deklarieren und mit dem Quadrat von a initialisieren
    Bruch x = a * a;

    // solange x < 1000
    while (x < Bruch(1000)) {
        // x mit a multiplizieren und ausgeben
        x *= a;
        x.print();
    }   
}

